package zrazumovskiy;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


public class DrawPanel extends JPanel
{
	private int numberOfPoints;
	private boolean normalize;
	
	private double x0;
	private double xN;
	private double y0;
	private double yN;

	//// The  coordinates in frame context,  the real values and the scales
	private double xScale;	
	private double yScale;	


	//// The coordinates of the grid, the real values of the grid and the step
	int[] xGrid;	
	int[] yGrid;			
	
	double[] xGridValue;	
	double[] yGridValue;

	private double xStep;	
	private double yStep;

	//// the vector field
	private double[][] Vx;
	private double[][] Vy;			

	//// The coordinates of the panel
	private int xFirstPoint = 100;
	private int yFirstPoint = 0;
	private int xLastPoint = 500;
	private int yLastPoint = 400;

	private int cellSize;

	private boolean clicked = false;
	private boolean warning = false;	
	private ArrayList linesCoords = new ArrayList();

	private int xClick;
	private int yClick;

	private int[] xContour;
	private int[] yContour;
	private boolean contour;

	private ArrayList[] animationLineCoords;
	private boolean animationInProgress;

	private Color[] colorArray = new Color[5];
	private ArrayList colorIndex = new ArrayList();
	private int activeColor = 0;

	//************************************** Constructor and Setting the parameters from Applet **************************************//
	public DrawPanel(int numberOfPoints, double x0, double xN, double y0, double yN, double[][] Vx, double[][] Vy, boolean normalize)
	{
		setParameters(numberOfPoints, x0, xN, y0, yN, Vx, Vy, normalize);		

		colorArray[0] = Color.blue;
		colorArray[1] = new Color(0, 64, 0);
		colorArray[2] = new Color(128, 0, 128);
		colorArray[3] = new Color(128, 64, 0);
		colorArray[4] = new Color(0, 64, 64);
	}

	public void setParameters(int numberOfPoints, double x0, double xN, double y0, double yN, double[][] Vx, double[][] Vy, boolean normalize)
	{
		this.numberOfPoints = numberOfPoints;
		this.normalize = normalize;
		this.Vx = Vx;
		this.Vy = Vy;

		this.cellSize = (this.yLastPoint - this.yFirstPoint) / this.numberOfPoints;   

		this.x0 = x0;
		this.xN = xN;
		this.y0 = y0;
		this.yN = yN;		

		this.xScale = Utils.getScale((xN - x0), (xLastPoint - xFirstPoint));
		this.yScale = Utils.getScale((yN - y0), (yLastPoint - yFirstPoint));

		this.xStep = (xLastPoint - xFirstPoint) / numberOfPoints;
		this.yStep = (yLastPoint - yFirstPoint) / numberOfPoints;

		this.setGrid();
		this.setGridValues();
	}
	
	public void setV(double[][] Vx, double[][] Vy)
	{
		this.Vx = Vx;
		this.Vy = Vy;
	}
	

	//*************************************************************************************************//

	//************************************** Calculating and drawing the grid **************************************//
	private void setGrid()
	{
		xGrid = new int[numberOfPoints];
		yGrid = new int[numberOfPoints];	

		for(int i = 0; i < numberOfPoints; i++)
		{
			xGrid[i] = xFirstPoint + (int)(xStep * i);
			yGrid[i] = yFirstPoint + (int)(yStep * i);			
		}
	}

	private void setGridValues()
	{
		xGridValue = new double[numberOfPoints];
		yGridValue = new double[numberOfPoints];		

		for(int i = 0; i < numberOfPoints; i++)
		{
			xGridValue[i] = (xGrid[i] - xFirstPoint) / xScale + x0;
			yGridValue[i] = (yLastPoint - yGrid[i]) / yScale + y0;
		}
	}

	private void drawGrid(Graphics g)
	{
		for(int i = 0; i < numberOfPoints; i++)
		{
			g.setColor(Color.gray);

			g.drawLine(xGrid[i], yFirstPoint, xGrid[i], yLastPoint);
			g.drawLine(xFirstPoint, yGrid[i], xLastPoint, yGrid[i]);

			String xStrValue = Float.toString((float)xGridValue[i]);
			String yStrValue = Float.toString((float)yGridValue[i]);
			
			if( i % 2 == 0 )
			{
				g.drawString(xStrValue, xGrid[i] - (int)(xStep / 2), yLastPoint + (int)yStep);
				g.drawString(yStrValue, xFirstPoint - (int)(2 * xStep), yGrid[i]);			
			}
		}
		
	}
	//*************************************************************************************************//

	private void setClipStatus(Graphics g, boolean status)
	{
		if (status)
			g.setClip(xFirstPoint, yFirstPoint, xLastPoint-xFirstPoint, yLastPoint-yFirstPoint);	
		else
			g.setClip(null);	
	}
	//************************************** Drawing the vector field **************************************//
	private void drawVectorField(Graphics g)
	{
		setClipStatus(g, true);
		for(int i = 0; i < numberOfPoints; i++)
		{
			for(int j = 0; j < numberOfPoints; j++)
			{
				int xVectorPoint;
				int yVectorPoint;
				
				if( normalize )
				{
					xVectorPoint = (int)(xGrid[i] + Vx[i][numberOfPoints - j - 1]  * cellSize);
					yVectorPoint = (int)(yGrid[j] + yStep - Vy[i][numberOfPoints - j - 1] * cellSize);
				}
				else
				{
					xVectorPoint = (int) (xGrid[i] + Vx[i][numberOfPoints - j - 1] / xScale);
					yVectorPoint = (int) (yGrid[j] + yStep - Vy[i][numberOfPoints - j - 1] / yScale);
				}

				g.setColor(Color.red);
				g.drawLine(xGrid[i], yGrid[j] + (int)yStep, xVectorPoint, yVectorPoint);
			}
		}
		setClipStatus(g, false);	
	}

        public void normalizeVectorField()
        {
            if (this.normalize)
                Utils.normalizeVectorField(Vx, Vy);
        }        
        //*************************************************************************************************//

	//************************************** Methods for getting panel properties **************************************//
	public double getXScale()
	{
		return xScale;		
	}
	public double getYScale()
	{
		return yScale;		
	}
	public int getXFirstPoint()
	{
		return xFirstPoint;		
	}
	public int getYLastPoint()
	{
		return yLastPoint;
	}
	//*************************************************************************************************//

	//************************************** Handle click and Draw lines **************************************//	
	public void drawLines(ArrayList[] singleLineCoords)
	{
		this.clicked = true;
		ArrayList[] temp = {new ArrayList(singleLineCoords[0]), new ArrayList(singleLineCoords[1])};
		this.colorIndex.add(new Integer(activeColor));	
		this.linesCoords.add(temp);
		this.warning = false;
	}

	private void drawLines(Graphics g)
	{
		setClipStatus(g, true);
		for(int k = 0; k < linesCoords.size(); k++)
		{
			int currentColorIndex = ((Integer)this.colorIndex.get(k)).intValue();
			g.setColor(this.colorArray[currentColorIndex]);	

			ArrayList[] singleLineCoords = (ArrayList[])linesCoords.get(k);
			int size = singleLineCoords[0].size();

			double[] x = new double[size];
			double[] y = new double[size];

			for(int i = 0; i < size; i++)
			{
				x[i] = ((Double)singleLineCoords[0].get(i)).doubleValue();	
				y[i] = ((Double)singleLineCoords[1].get(i)).doubleValue();	
			}
	
			for(int i = 0; i < x.length - 1; i++)
			{
				int xCurr = (int)( (x[i] - x0) * xScale + xFirstPoint);
				int yCurr = (int)( yLastPoint - (y[i] - y0) * yScale);
				int xNext = (int)( (x[i + 1] - x0) * xScale + xFirstPoint);
				int yNext = (int)(yLastPoint - (y[i + 1] - y0) * yScale);
								
				g.drawLine(xCurr, yCurr, xNext, yNext);
			}
		}
		setClipStatus(g, false);
		g.setColor(Color.blue);
	}

	void changeColor()
	{
		activeColor++;
		if( activeColor == colorArray.length )
			activeColor = 0;
	}
// ---------------- Animation
        public void drawAnimationLines(ArrayList[] singleLineCoords)
	{
		this.animationInProgress = true;
		this.animationLineCoords = singleLineCoords;
		this.warning = false;
	}

	private void drawAnimationLines(Graphics g)
	{
		setClipStatus(g, true);
		
			int size = animationLineCoords[0].size();

			double[] x = new double[size];
			double[] y = new double[size];

			for(int i = 0; i < size; i++)
			{
				x[i] = ((Double)animationLineCoords[0].get(i)).doubleValue();	
				y[i] = ((Double)animationLineCoords[1].get(i)).doubleValue();	
			}
	
			g.setColor(Color.blue);
			for(int i = 0; i < x.length - 1; i++)
			{
				int xCurr = (int)( (x[i] - x0) * xScale + xFirstPoint);
				int yCurr = (int)( yLastPoint - (y[i] - y0) * yScale);
				int xNext = (int)( (x[i + 1] - x0) * xScale + xFirstPoint);
				int yNext = (int)(yLastPoint - (y[i + 1] - y0) * yScale);

				g.drawLine(xCurr, yCurr, xNext, yNext);
			}
		
		setClipStatus(g, false);
	}
// ---------------- End of Animation        
        
	public void clearLines()
	{
		this.linesCoords.clear();
		this.clicked = false;
		this.contour = false;	
  	              this.animationInProgress = false;
		this.colorIndex.clear();
		this.activeColor = 0; 
	}

	public void drawWarning(int x, int y)
	{
		this.warning = true;	
		this.xClick = x;
		this.yClick = y;
	}	
	//*************************************************************************************************//

	//******************************* Draw Contour *************************************************//		
	public void drawContour(ArrayList xCoord, ArrayList yCoord)
	{
		contour = true;
		int length = xCoord.size();
		xContour = new int[length];
		yContour = new int[length];
		
		for( int i = 0; i < length; i++ )
		{
			xContour[i] = ((Integer)xCoord.get(i)).intValue();		
			yContour[i] = ((Integer)yCoord.get(i)).intValue();		
		}						
	}

	private void drawContour(Graphics g)
	{
		setClipStatus(g, true);
		g.setColor(Color.black);
		for( int i = 0; i < xContour.length - 1; i++ )
			g.drawLine(xContour[i], yContour[i], xContour[i + 1], yContour[i + 1]);
		setClipStatus(g, false);	
	}

	//*************************************************************************************************//

	//******************************* Main Draw panel method ************************************//
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		g.setColor(Color.white);
		g.fillRect(xFirstPoint, yFirstPoint, (xLastPoint - xFirstPoint), (yLastPoint - yFirstPoint));
		drawGrid(g);
		drawVectorField(g);
		if( clicked )
			drawLines(g);
		if( warning )
			g.drawString("click on panel", xClick, yClick);	
		if( contour )
			drawContour(g);
                if(animationInProgress)
                        drawAnimationLines(g);
	}	
}












