package zrazumovskiy;

import java.util.*;

public class Integration
{
	private double[] x; 
	private double[] y; 
	private double[] S;
	
	private Spline xSpline;
	private Spline ySpline;	
	
	private Parser parser;
	
	private double sTrapzd;

	private	double EPS = 0.0001; // absolute precision for calculation of integrals

	public Integration(ArrayList x, ArrayList y, Parser parser)
	{
		this.parser = parser;
		int length = x.size();
		this.x = new double[length];
		this.y = new double[length];

		for( int i = 0; i < length; i++ )
		{
			this.x[i] = ((Double)x.get(i)).doubleValue();
			this.y[i] = ((Double)y.get(i)).doubleValue();
		}
		//this.x[length-1]=this.x[0];
		//this.y[length-1]=this.y[0];
		calculateSKoeffs();		

		xSpline = new Spline(this.S, this.x);
		ySpline = new Spline(this.S, this.y);
	}
	
	public double getCirculationValue()
	{
		double integralValue = 0;
		for( int i = 1; i < x.length; i++ )
		{
			integralValue = integralValue + getIntervalIntegralQSimp(i);
		}
		
		return integralValue;
	}
	private double F(double s, int k)
	{
	  double result;
	  double xtmp, ytmp;
	  xtmp = xSpline.getInterpolatedValue(s, k);	
	  ytmp = ySpline.getInterpolatedValue(s, k);
	  result=parser.getFValue(xtmp, ytmp)*xSpline.getDerivative(s,k)+
	  		parser.getGValue(xtmp, ytmp)*ySpline.getDerivative(s,k);
	  return result;
	}
	
	private double trapzd(int k, int n)
	{
	  int it = 1;
	  double sum = 0.;
	  if (n==1)
	  	{ sTrapzd=0.5*(S[k]-S[k-1])*(F(S[k-1],k)+F(S[k],k));}
	  else
	  	{
		 for (int j=1; j< n-1; j++) it <<=1;
		 double tnm = it;
		 double del = (S[k]-S[k-1])/tnm;
		 double xtmp = S[k-1] + 0.5* del;
		 for (int j=1; j<=it; j++, xtmp+=del) sum +=F(xtmp, k);
		 sTrapzd = 0.5 *(sTrapzd+(S[k]-S[k-1])*sum/tnm);
		}	
	return sTrapzd;	
	}
	private double getIntervalIntegralQSimp( int k)
	{
		int JMAX = 20;
		double s=0.0;
		double st=0.0;
		double ost=0.0;
		double os=0.0;
		boolean b = true;
		boolean error = false;
	
		for (int j=1; (j<=JMAX) && b; j++)
		{
			st=trapzd(k, j);
			s=(4.0*st-ost)/3.0;
			if (j>5)
			 if ((Math.abs(s-os)< EPS) || (s == 0. && os ==0.)) b = false;
			os = s;
			ost = st; 
			if (j==JMAX) error=true;
		}
		return s;
	}
	
	public void calculateSKoeffs()
	{
		int length = x.length;
		S = new double[length];
		S[0] = 0;

		for( int i = 1; i < length; i++)
		{
			S[i] = S[i - 1] + Math.sqrt( (x[i] - x[i - 1]) * (x[i] - x[i - 1]) + (y[i] - y[i - 1]) * (y[i] - y[i - 1]) );
		}
	}	

	public ArrayList[] getInterpolatedPoints(int M)	
	{
		ArrayList xPoints = new ArrayList();
		ArrayList yPoints = new ArrayList();

		double x, y, s, deltaSm;

		for( int i = 1; i < S.length; i++ )
		{
			deltaSm=(S[i]-S[i-1])/M;
			for ( int j = 0; j < M; j++)
			{
				s=S[i-1]+j*deltaSm;
                        	x = xSpline.getInterpolatedValue(s, i);
	                        y = ySpline.getInterpolatedValue(s, i);
			
				xPoints.add(new Double(x));
				yPoints.add(new Double(y));
        	        }

		}

                x = xSpline.getInterpolatedValue(0, 1);
	        y = ySpline.getInterpolatedValue(0, 1);
			
		xPoints.add(new Double(x));
		yPoints.add(new Double(y));


		ArrayList[] result = {xPoints, yPoints};

		return result;
	}
	public void setInterpolationType(boolean interpolation)
	{
		xSpline.setInterpolationType(interpolation);
		ySpline.setInterpolationType(interpolation);
	}
	public boolean getInterpolationType()
	{
		return xSpline.getInterpolationType();
	}
}
