package zrazumovskiy;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class MyApplet extends JApplet
{
	private Parser parser;
	private Integration integration;
	private Integration firstContour;

	//// Fields values and calculation parameters
	private String expressionF = "cos(x-t)";
	private String expressionG = "sin(x-t)";
	private int numberOfPoints = 20;
	private double x0 = -5;
	private double xN = 5;
	private double y0 = -5;
	private double yN = 5;		 
	private double t = 0;
	private boolean normalize = true;
	private double integrStep = 2;
	private byte lineType = 0;
	
	private int mKoeffStreakLines = 10;
	private double tInitStreakLines = 0;

	private ArrayList xContour = new ArrayList();
	private ArrayList yContour = new ArrayList();
	private boolean contourClosed = false;
	private ArrayList xContourValues = new ArrayList();
	private ArrayList yContourValues = new ArrayList();

	//// Vector Field
	private double[][] Vx;
	private double[][] Vy;

	private ArrayList xStreamLinesClicks = new ArrayList(); 
	private ArrayList yStreamLinesClicks = new ArrayList(); 

	//// GUI Components
	Box vBox = Box.createVerticalBox();
	Box hBox = Box.createHorizontalBox();
	private JTextField tfInputF; 
	private JTextField tfInputG; 
	private JTextField tfx0;
	private JTextField tfxN;
	private JTextField tfy0;
	private JTextField tfyN;
	private JTextField tfNumberOfPoints;
	private JTextField tfTime;
	private JButton buttonDraw = new JButton("draw");
	private JCheckBox chkBoxNormalize = new JCheckBox("Normalize", true);	
	private JCheckBox chkBoxStreamLinesTrack = new JCheckBox("Change stream lines in time", false);
	private JComboBox cbLinesType = new JComboBox();
	private JLabel labWarning = new JLabel("              ");	
	private JTextField tfPathStreakLinesRedrawTime = new JTextField("0.1");

	private Box hContItegrBox = Box.createHorizontalBox();
	private JButton btnClearContour = new JButton("clear");
	private JButton btnCloseContour = new JButton("close");
	private JButton btnCirculation = new JButton("circulation:");
	private JButton btnInterpolate = new JButton("use interpolated");
	private JLabel labCirculation = new JLabel();
	private JTextField tfContNDiv = new JTextField("0", 2);
	private Box hContNDivBox = Box.createHorizontalBox(); 
	
	private DrawPanel panel;	

	private Container cont;

	//************************************** Applet initalization **************************************//
	public void init()
 	{
	 	cont = getContentPane();

		x0 = Double.parseDouble(getParameter("x0"));
		xN = Double.parseDouble(getParameter("xN"));
		y0 = Double.parseDouble(getParameter("y0"));
		yN = Double.parseDouble(getParameter("yN"));
		t = Double.parseDouble(getParameter("t"));
		numberOfPoints = Integer.parseInt(getParameter("numberOfPoints"));
		normalize = chkBoxNormalize.isSelected();

		createGUI();
		setInputFieldsValues();

		parser = new Parser();
		calculateVectorField();
		
		panel = new DrawPanel(this.numberOfPoints, x0, xN, y0, yN, Vx, Vy, normalize);
		addPanelListener();
		cont.add(panel); 

		cont.add(hBox,BorderLayout.NORTH);
	}
	//*************************************************************************************************//

	//************************************** Reading users input **************************************//
	private void getInputFieldsValues()
	{
		expressionF = tfInputF.getText();
		expressionG = tfInputG.getText();
		x0 = Double.parseDouble(tfx0.getText());
		xN = Double.parseDouble(tfxN.getText());
		y0 = Double.parseDouble(tfy0.getText());
		yN = Double.parseDouble(tfyN.getText());
		t = Double.parseDouble(tfTime.getText());
		numberOfPoints = Integer.parseInt(tfNumberOfPoints.getText());
		normalize = chkBoxNormalize.isSelected();
	}

	//// called once in initialization
	private void setInputFieldsValues()
	{
		tfInputF.setText(expressionF);
		tfInputG.setText(expressionG);
		tfx0.setText(Double.toString(x0));
		tfxN.setText(Double.toString(xN));
		tfy0.setText(Double.toString(y0));
		tfyN.setText(Double.toString(yN));
		tfNumberOfPoints.setText(Integer.toString(numberOfPoints));
		tfTime.setText(Double.toString(t));

	}

	//*************************************************************************************************//

	//************************************** GUI initialization **************************************//
	private void createGUI()
	{
		createContIntegrGUIPanel();			// Create the integration GUI panel  ( hContItegrBox )	
		setIntegrationGUIVisible(false);		// Hide the integration GUI 

		tfPathStreakLinesRedrawTime.setEnabled(false);

		tfInputF = new JTextField(expressionF);		
		tfInputG = new JTextField(expressionG);
		tfx0 = new JTextField(Double.toString(x0));
		tfxN = new JTextField(Double.toString(xN));
		tfy0 = new JTextField(Double.toString(y0));
		tfyN = new JTextField(Double.toString(yN));
		tfNumberOfPoints = new JTextField(Integer.toString(numberOfPoints));
		tfTime = new JTextField(Double.toString(t));

		Box hBoxF = Box.createHorizontalBox();
		hBoxF.add(new JLabel("Input Vx:"));
		hBoxF.add(tfInputF);

		Box hBoxG = Box.createHorizontalBox();
		hBoxG.add(new JLabel("Input Vy:"));
	hBoxG.add(tfInputG);

		Box hBoxX = Box.createHorizontalBox();
		hBoxX.add(new JLabel("Input x0:"));
		hBoxX.add(tfx0);
		hBoxX.add(new JLabel("Input xN:"));
		hBoxX.add(tfxN);

		Box hBoxY = Box.createHorizontalBox();
		hBoxY.add(new JLabel("Input y0:"));
		hBoxY.add(tfy0);
		hBoxY.add(new JLabel("Input yN:"));
		hBoxY.add(tfyN);

		Box hBoxT_N = Box.createHorizontalBox();
		hBoxT_N.add(new JLabel("Input t:"));
		hBoxT_N.add(tfTime);
		hBoxT_N.add(new JLabel("Input number of points:"));
		hBoxT_N.add(tfNumberOfPoints);

		cbLinesType.addItem("Draw Stream Lines");
		cbLinesType.addItem("Draw Path Lines");
		cbLinesType.addItem("Draw Streak Lines");
		cbLinesType.addItem("Draw Contour");

		Box hBoxNorm_LinesType = Box.createHorizontalBox();
		hBoxNorm_LinesType.add(chkBoxNormalize);
		hBoxNorm_LinesType.add(Box.createHorizontalStrut(10));
		hBoxNorm_LinesType.add(chkBoxStreamLinesTrack);
		hBoxNorm_LinesType.add(Box.createHorizontalStrut(10));
		hBoxNorm_LinesType.add(new JLabel("Input redraw time interval:")); 		
		hBoxNorm_LinesType.add(tfPathStreakLinesRedrawTime); 
		hBoxNorm_LinesType.add(Box.createHorizontalStrut(10));
		hBoxNorm_LinesType.add(cbLinesType);

		//		vBox.add(hBoxF);	
		//		vBox.add(hBoxG);	
		vBox.add(Box.createVerticalStrut(20));
		vBox.add(hBoxX);
		vBox.add(hBoxY);
		vBox.add(hBoxT_N);
		vBox.add(hBoxNorm_LinesType);
		vBox.add(labWarning);
		vBox.add(hContItegrBox);
		vBox.add(hContNDivBox);		
		vBox.add(Box.createVerticalStrut(20));		
		
	
		buttonDraw.addActionListener( new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if( lineType == 3 && contourClosed )
					{
						String tFinalStr = tfTime.getText();	
						double tFinal = Double.parseDouble(tFinalStr);
						double xScale = panel.getXScale();
						double yScale = panel.getYScale();                                               
  			                                          boolean intType = integration.getInterpolationType();
						int nDiv = Integer.parseInt(tfContNDiv.getText());
						ArrayList[] contourPoints = firstContour.getInterpolatedPoints(nDiv+1);
						
						xContourValues = contourPoints[0];
						yContourValues = contourPoints[1];
						
						contourPoints = parser.calculateContourPoints(xContourValues, yContourValues, t, tFinal, integrStep);
						xContourValues = contourPoints[0];
						yContourValues = contourPoints[1];
						
						integration = new Integration(xContourValues, yContourValues, parser);
						integration.setInterpolationType(intType);
							
 						drawInterpolatedPoints(); 

						getInputFieldsValues();
						setInputFieldsValues();
						calculateVectorField();
						panel.setParameters(numberOfPoints, x0, xN, y0, yN, Vx, Vy, normalize);

						labCirculation.setText("");
						panel.repaint();
					}
					else 
					{
						clearContour();

						if( lineType == 0 && chkBoxStreamLinesTrack.isSelected() )
						{
							panel.changeColor();
							ArrayList[] result = null;		
							String tFinalStr = tfTime.getText();	
							double tFinal = Double.parseDouble(tFinalStr);
							double xScale = panel.getXScale();
							double yScale = panel.getYScale();                                               
							double integrKoeff = integrStep / xScale;
							ArrayList[] newPoints = parser.calculateContourPoints(xStreamLinesClicks, yStreamLinesClicks, t, tFinal, integrStep);

							for( int i = 0; i < newPoints[0].size(); i++ )
							{
								double xInit = ((Double)newPoints[0].get(i)).doubleValue();		
								double yInit = ((Double)newPoints[1].get(i)).doubleValue();		

								getInputFieldsValues();
								setInputFieldsValues();
								calculateVectorField();
								panel.setParameters(numberOfPoints, x0, xN, y0, yN, Vx, Vy, normalize);

								result = parser.calculateStreamLines(xInit, yInit, integrKoeff, xScale, yScale);
								panel.drawLines(result);
							}	
	
							panel.repaint();

						}
						else
						{	
							getInputFieldsValues();
							setInputFieldsValues();
							calculateVectorField();
							clearContour();
							panel.setParameters(numberOfPoints, x0, xN, y0, yN, Vx, Vy, normalize);
							panel.clearLines();
							panel.repaint();			
						}
					}
				}
			}
				
				    	    );
		
		cbLinesType.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					lineType = (byte)cbLinesType.getSelectedIndex();
					clearContour();
					panel.clearLines();
					xContourValues.clear();
					yContourValues.clear();

					if( lineType == 0 )
					{
						tfTime.setEnabled(true);		
						chkBoxStreamLinesTrack.setEnabled(true);
						chkBoxStreamLinesTrack.setVisible(true);
						tfPathStreakLinesRedrawTime.setEnabled(false);
					}
					else
					{
						tfPathStreakLinesRedrawTime.setEnabled(true);
						tfTime.setEnabled(false);			
						chkBoxStreamLinesTrack.setEnabled(false);
						chkBoxStreamLinesTrack.setVisible(false);
					}	

					if( lineType == 3 )
					{
						tfPathStreakLinesRedrawTime.setEnabled(false);
						tfTime.setEnabled(true);		
						setIntegrationGUIVisible(true);
					}
					else	
						setIntegrationGUIVisible(false);
				}
			}
					
					  );

		hBox.add(Box.createHorizontalStrut(100));  
		hBox.add(vBox);
		hBox.add(buttonDraw);
	}
	//*************************************************************************************************//

	//************************************** Vector field Calculations **************************************//
	private void calculateVectorField()
	{
		labWarning.setText("              ");

		parser.setParameters(this.numberOfPoints, this.x0, this.xN, this.y0, this.yN, this.t);

		Vx = new double[numberOfPoints][numberOfPoints];
		Vy = new double[numberOfPoints][numberOfPoints];

		try
		{
			Vx = parser.calculateF();	
			Vy = parser.calculateG();	
		}
		catch(UndefinedValueException e)
		{
			double xExceptCoord = e.getXCoord();
			double yExceptCoord = e.getYCoord();

			labWarning.setForeground(Color.red);
			labWarning.setText("Undefined Value at: " +  (new Double(xExceptCoord)).toString() + " " + (new Double(yExceptCoord)).toString() );
		} 
		
		if( normalize )
			Utils.normalizeVectorField(Vx, Vy);
	}
	//*************************************************************************************************//

	//********************************** Contour calculations GUI *********************//
	private boolean contourIsClosed()
	{
		int length = xContour.size() - 1;
		int xFirstPoint = ((Integer)xContour.get(0)).intValue();
		int yFirstPoint = ((Integer)yContour.get(0)).intValue();
		int xLastPoint = ((Integer)xContour.get(length)).intValue();
		int yLastPoint = ((Integer)yContour.get(length)).intValue();
		
		if( length > 1 && Math.abs(xLastPoint - xFirstPoint) <= 1 && Math.abs(yLastPoint - yFirstPoint) <= 1 )
			return true;
		else 
			return false;
	}

	private void clearContour()
	{
		xContour.clear();
		yContour.clear();
		xContourValues.clear();
		yContourValues.clear();
		contourClosed = false;
		labCirculation.setText("");
	}

	private void getContourPointsValues(double xScale, double yScale)
	{
		xContourValues.clear();
		yContourValues.clear();
		for( int i = 0; i < xContour.size(); i++)
		{
			int xClick = ((Integer)xContour.get(i)).intValue();
			int yClick = ((Integer)yContour.get(i)).intValue();
			
			double x = (xClick - panel.getXFirstPoint()) / xScale + x0;
			double y = (panel.getYLastPoint() - yClick) / yScale + y0;

			xContourValues.add(new Double(x));
			yContourValues.add(new Double(y));
		}
	}

	private void createContIntegrGUIPanel()
	{
		btnCloseContour.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if( xContour.size() > 2 )
					{
						xContour.add(xContour.get(0));
						yContour.add(yContour.get(0));
						contourClosed = true;
						panel.drawContour(xContour, yContour);
						double xScale = panel.getXScale();
						double yScale = panel.getYScale();
						getContourPointsValues(xScale, yScale);
						btnCirculation.setVisible(true);
						btnCirculation.setEnabled(true);	
						btnInterpolate.setVisible(true);
						btnInterpolate.setEnabled(true);	
                                                btnInterpolate.setText("use interpolated");
						
						firstContour = new Integration(xContourValues, yContourValues, parser);
						integration = firstContour;
						
						panel.repaint();
						
					}
					else
						labCirculation.setText("unable to close contour with "  + xContour.size() + " points");
				}
			}
						);

		btnClearContour.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					clearContour();
					panel.clearLines();
					btnCirculation.setVisible(false);
					btnCirculation.setEnabled(false);	
					btnInterpolate.setVisible(false);
					btnInterpolate.setEnabled(false);	
					panel.repaint();	
				}
			}
						);
		btnCirculation.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					getCirculation();
					panel.repaint();
				}
			}
						);

		btnInterpolate.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					integration.setInterpolationType(! integration.getInterpolationType());
					drawInterpolatedPoints();
					if (integration.getInterpolationType() )
						btnInterpolate.setText("use original");
					else	
						btnInterpolate.setText("use interpolated");					
				}
			}	
						);

		hContItegrBox.add(new JLabel("Contour:"));
		hContItegrBox.add(Box.createHorizontalStrut(5));
		hContItegrBox.add(btnCloseContour);
		hContItegrBox.add(btnClearContour);
		hContItegrBox.add(btnInterpolate);
		hContItegrBox.add(Box.createHorizontalStrut(5));		
		hContItegrBox.add(btnCirculation);
		hContItegrBox.add(labCirculation);
		hContNDivBox.add(new JLabel("# of points added per side to advance in time"));
		hContNDivBox.add(Box.createHorizontalStrut(5));		
		hContNDivBox.add(tfContNDiv);
		
	}
	private void setIntegrationGUIVisible(boolean b)
	{
		hContItegrBox.setVisible(b);
		hContNDivBox.setVisible(b);		
		btnCloseContour.setVisible(b);
		btnCloseContour.setEnabled(b);
		btnClearContour.setVisible(b);
		btnClearContour.setEnabled(b);
		btnCirculation.setVisible(false);
		btnCirculation.setEnabled(false);
		btnInterpolate.setVisible(false);
		btnInterpolate.setEnabled(false);	
		labCirculation.setText("");
	}

	private void getCirculation()
	{
		double circulation = integration.getCirculationValue();
		circulation = (double) ( (int) (circulation*10000.0 + 0.5))/10000.0;
		labCirculation.setText(Double.toString(circulation));
	}

	private void drawInterpolatedPoints()
	{
		ArrayList[] contourPoints = integration.getInterpolatedPoints(10);
		xContourValues = contourPoints[0];
		yContourValues = contourPoints[1];	
		panel.drawLines(contourPoints);
		panel.repaint();	
	}
	//*************************************************************************************************//
	
	//************************************** Panel Listener **************************************//
	private void addPanelListener()
	{
		panel.addMouseListener(new MouseAdapter()
			{
				public void mouseClicked(MouseEvent e)
				{
					int xClick = e.getX();
					int yClick = e.getY();
					double xScale = panel.getXScale();
					double yScale = panel.getYScale();
					double xInitCond = (xClick - panel.getXFirstPoint()) / xScale + x0;
					double yInitCond = (panel.getYLastPoint() - yClick) / yScale + y0;

					
					if( (xInitCond > x0) && (xInitCond < xN) && (yInitCond > y0) && (yInitCond < yN))	
					{
						double integrKoeff = integrStep / xScale; 
						ArrayList[] result = null;
                                                
			                                         labWarning.setText("wait...");
                                          		             labWarning.paintImmediately(0, 0, labWarning.getWidth(), labWarning.getHeight());                                                             

						if( lineType == 0 )	
						{
							if( chkBoxStreamLinesTrack.isSelected() )
							{	
								xStreamLinesClicks.add(new Double(xInitCond));
								yStreamLinesClicks.add(new Double(yInitCond));
							}
							else
							{
								xStreamLinesClicks.clear();	
								xStreamLinesClicks.clear();	
							}

  					                            result = parser.calculateStreamLines(xInitCond, yInitCond, integrKoeff, xScale, yScale);  
                                                  				panel.drawLines(result);  
						}
						if( lineType == 1 )
						{
							String tDeltaStr = tfPathStreakLinesRedrawTime.getText();	
							double tDelta = Double.parseDouble(tDeltaStr);

							result = parser.calculatePathLines(xInitCond,
yInitCond, t, integrKoeff, xScale, yScale, tDelta, panel);
							panel.drawLines(result);							
						}
						if( lineType == 2 )
						{
							String tDeltaStr =  tfPathStreakLinesRedrawTime.getText();	
							double tDelta = Double.parseDouble(tDeltaStr);
							result = parser.calculateStreakLines(xInitCond, yInitCond, t, tInitStreakLines, integrKoeff, mKoeffStreakLines, xScale, yScale, panel, tDelta);
							panel.drawLines(result);							
						}
						if( lineType == 3 )
						{		
							if( !contourClosed )							
							{
								xContour.add(new Integer(xClick));
								yContour.add(new Integer(yClick));
								panel.drawContour(xContour, yContour);	
	
								if( contourIsClosed() )
								{
									contourClosed = true;
									getContourPointsValues(xScale, yScale);
									btnCirculation.setVisible(true);
									btnCirculation.setEnabled(true);
									btnInterpolate.setVisible(true);
									btnInterpolate.setEnabled(true);	
									btnInterpolate.setText("use interpolated");
									
									firstContour = new Integration(xContourValues, yContourValues, parser);
									integration = firstContour;
								}							
											
							}
							else
								labCirculation.setText("contour is closed");		
						}	


					}
					else
					{
						panel.drawWarning(xClick, yClick);
					}

                                        labWarning.setText("             ");                                          
					panel.repaint();
				}
			}	
					);
	}
	//*************************************************************************************************//

}









