package zrazumovskiy;

import java.util.*;


public class Parser
{
	double[][] Vx;
	double[][] Vy;

	private int numberOfPoints;
	private double x0;
	private double y0;
	private double xN;
	private double yN; 
	private double t = 0;
	private double xStep;
	private double yStep;

	private FunctionVx parserF = new FunctionVx();
	private FunctionVy parserG = new FunctionVy();

	RungeKutta rk;
	
	public Parser()
	{
		parserF.addVariable("x",0);
		parserF.addVariable("y",0);
		parserF.addVariable("t",0);

		parserG.addVariable("x",0);
		parserG.addVariable("y",0);
		parserG.addVariable("t",0);

	}

// ===========================================================
        
	public void setParameters(int numberOfPoints, double x0, double xN, double y0, double yN, double t)
	{
		this.numberOfPoints = numberOfPoints;	
		this.x0 = x0;
		this.xN = xN;
		this.y0 = y0;
		this.yN = yN;
		this.t = t;	

		xStep = Utils.getStep(this.x0, this.xN, this.numberOfPoints);	
		yStep = Utils.getStep(this.y0, this.yN, this.numberOfPoints);	
	}
	
	public void setTime(double t)
	{
		this.t = t;
	}
	private void setParserFValues(double x, double y)
	{
		parserF.addVariable("x", x);
		parserF.addVariable("y", y);
	}

	private void setParserGValues(double x, double y)
	{
		parserG.addVariable("x", x);
		parserG.addVariable("y", y);
	}

	//***********************************************//
		
	//******************* get F, G values ************//	
	public double getFValue(double x, double y)
	{
		setParserFValues(x, y);
		return (parserF.getValue());
	}
	public double getGValue(double x, double y)
	{
		setParserGValues(x, y);
		return (parserG.getValue());
	}

	//******************* Vector Field Calculations ************//
	public double[][] calculateF() throws UndefinedValueException
	{
		Vx = new double[numberOfPoints][numberOfPoints];

		boolean undefinedValue = false;

		parserF.addVariable("x", x0);
		parserF.addVariable("y", y0);
		parserF.addVariable("t", t);
		
		for(int i = 0; i < numberOfPoints; i++)
		{
			parserF.addVariable("x", x0 + i * xStep);
			for(int j = 0; j < numberOfPoints; j++)
			{
				parserF.addVariable("y", y0 + j * yStep);
				Vx[i][j] = parserF.getValue();

				if( valueIsUndefined(Vx[i][j]) )
					throw new UndefinedValueException(x0 + i * xStep, y0 + j * yStep);					
			}
		}

		return Vx;		
	}

	public double[][] calculateG() throws UndefinedValueException 
	{
		Vy = new double[numberOfPoints][numberOfPoints];

		parserG.addVariable("x", x0);
		parserG.addVariable("y", y0);
		parserG.addVariable("t", t);

		for(int i = 0; i < numberOfPoints; i++)
		{
			parserG.addVariable("x", x0 + i * xStep);
			for(int j = 0; j < numberOfPoints; j++)
			{			
				parserG.addVariable("y", y0 + j * yStep);
				Vy[i][j] = parserG.getValue();

				if( valueIsUndefined(Vy[i][j]) )
					throw new UndefinedValueException(x0 + i * xStep, y0 + j * yStep);				
			}
		}

		return Vy;		
	}

	private boolean valueIsUndefined(double Vxy)
	{
		if( Double.isNaN(Vxy) || Double.isInfinite(Vxy)  )
			return true;
		
		return false;
	} 	

	//************************************************************************//

	//******************* Lines Calculations *********************//
	public ArrayList[] calculateStreamLines(double xInit, double yInit, double integrKoeff, double xScale, double yScale)	
	{
		parserF.addVariable("t", this.t);
		parserG.addVariable("t", this.t);
		rk = new RungeKutta(x0, xN, y0, yN, parserF, parserG);
		ArrayList[] result = rk.calculateStreamLines(xInit, yInit, integrKoeff, xScale, yScale);
		return result;
	}

	public ArrayList[] calculatePathLines(double xInit, double yInit, double t, double integrKoeff,
	double xScale, double yScale, double tDelta, DrawPanel panel)
	{
		rk = new RungeKutta(x0, xN, y0, yN, parserF, parserG, this, panel, tDelta);
		ArrayList[] result = rk.calculatePathLines(xInit, yInit, t, Double.POSITIVE_INFINITY,
		integrKoeff, xScale, yScale,true);
		return result;
	}

	public ArrayList[] calculateStreakLines(double xInit, double yInit, double tCurr, double tStart, double integrKoeff, int mStep, double xScale, double yScale, DrawPanel panel, double tDelta)
	{	
		rk = new RungeKutta(x0, xN, y0, yN, parserF, parserG, this, panel, tDelta);
		ArrayList[] result = rk.calculateStreakLines(xInit, yInit, tCurr, tStart, integrKoeff, mStep, xScale, yScale);
		return result;
	}
	
	public ArrayList[] calculateContourPoints(ArrayList xCoord, ArrayList yCoord, double t0, double tFinal, double integrKoeff )
	{
		rk = new RungeKutta(x0, xN, y0, yN, parserF, parserG);
		ArrayList[] result = rk.calculateContourPoints(xCoord, yCoord, t0, tFinal, integrKoeff);
		return result;		
	}
	
	
}







