package zrazumovskiy;


public class Spline
{
	private double [] y; //stores y-values
	private double [] x; //stores x-values
	private double [] y2prime; //stores second derivatives
	private int N;
	private byte TypeOfSpline = 2;
	/* Type of spline to calculate:
	0 - Fixed first derivatives at the bounadries; 
	1 - Natural spline (second derivatives are zero at the boundaries); 
	2 - No splines, use linear interpolation
	
	*/
	public Spline(double [] xVal, double [] yVal)
	/*
	given arrays of values y[i] at points x[i] calculates array of the second derivatives 
	y2prime[i]
	*/
	{
		double sig, p, un, qn, yprime=0;
		N = xVal.length;
		y = yVal; //store y-values
 		x = xVal; //store x-values	
		
		// if (TypeOfSpline == 2) return;
		// No calculations are necessary for linear interpolation
		
		y2prime = new double [N]; 

		double [] tmpArr = new double [N];
		
		// 
                if (TypeOfSpline == 0)
                {
                    yprime = (y[1]-y[0])/(x[1] -x[0]);
                    y2prime[0] = -0.5;
                    tmpArr[0]=(3.0/(x[1]-x[0]))*((y[1]-y[0])/(x[1]-x[0])-yprime);
                }
                else
                {
                    y2prime[0]=0.;
                    tmpArr[0]=0.;
                }
		
		for (int i=1; i<=N-2; i++)
		{
			sig = (x[i]-x[i-1])/(x[i+1]-x[i-1]);
			p = sig* y2prime[i-1] +2.0;
			y2prime[i] = (sig - 1.0)/p;
			tmpArr[i] = (y[i+1]-y[i])/(x[i+1]-x[i])  -  (y[i]-y[i-1])/(x[i]-x[i-1]) ;
			tmpArr[i] = (6* tmpArr[i] / (x[i+1]-x[i-1]) - sig* tmpArr[i-1])/p;
		}
                
                if (TypeOfSpline == 0)
                {
                    qn = 0.5;
                    un = (3.0 /(x[N-1] -x[N-2]))*(yprime - (y[N-1]-y[N-2])/(x[N-1] -x[N-2]));
                }
                else
                {
                    qn=0;
                    un=0;
                }
                y2prime[N-1] = (un - qn * tmpArr[N-2])/(qn * y2prime[N-2]+1.0);

	
		for (int i=N-2; i>=0; i--)
		{
			y2prime[i]= y2prime[i]*y2prime[i+1]+tmpArr[i];
		}
	}
	public double getInterpolatedValue(double xVal, int j)
	/*
	given point x and the index of the interval, such that x(j-1)<x<x(j), returns
	interpolated value y(x)
	*/
	{
	
	if (TypeOfSpline == 2) return LinearInterpolation(xVal,j);
	double h;
	double A;
	double B;
	double yAnswer;
	
	h = x[j]-x[j-1];
	A = (x[j]-xVal)/h;
	B = (xVal - x[j-1])/h;
	
	if (h != 0)
	{
            yAnswer = A* y[j-1] + B*y[j] +((A*A*A-A)*y2prime[j-1]+(B*B*B-B)*y2prime[j])*(h*h)/6.0; 	
	}
	else
	{
	    yAnswer = y[j];
	}
        return yAnswer;
	}
	private double LinearInterpolation(double xVal, int j)
	{
	double h;
	double A;
	double B;
	double yAnswer;
	
	h = x[j]-x[j-1];
	A = (x[j]-xVal)/h;
	B = (xVal - x[j-1])/h;
	
	yAnswer = A* y[j-1] + B*y[j]; 	
	
	return yAnswer;
	}
	
	public double getDerivative(double xVal, int j)
	{
	double h;
	double A;
	double B;
	double result;
	h = x[j]-x[j-1];
	
	if (TypeOfSpline == 2) return (y[j]-y[j-1])/h;

	A = (x[j]-xVal)/h;
	B = (xVal - x[j-1])/h;
	
	if (h != 0)
	{ 
	    result =  (y[j]-y[j-1])/h - (3.*A*A-1.)*h* y2prime[j-1]/6.+
			(3.*B*B-1.)*h* y2prime[j]/6. ; 	
	}
	else
	{
	    result = 0.;
	}
	return result;
	}
	
	public void setInterpolationType(boolean interpolation)
	{
		if (interpolation) 
			TypeOfSpline = 0; 
		else 
			TypeOfSpline = 2;
	}
	public boolean getInterpolationType()
	{
		return (TypeOfSpline==0);
	}
}
